@@ -508,7 +508,7 @@ impl ClientRequest {
                         c.value().as_bytes(),
                         crate::http::helpers::USERINFO,
                     );
-                    let _ = write!(&mut cookie, "; {}={}", name, value);
+                    let _ = write!(cookie, "; {}={}", name, value);
                 }
                 self.head.headers.insert(
                     header::COOKIE,
