@@ -177,6 +177,7 @@ where
                                     let item = if let Some(item) = msg.head().take_io() {
                                         item
                                     } else {
+                                        log::trace!("Handler service consumed io, exit");
                                         return Poll::Ready(Ok(()));
                                     };
 
@@ -212,6 +213,10 @@ where
                                     buf.extend_from_slice(b"HTTP/1.1 100 Continue\r\n\r\n")
                                 });
                                 if result.is_err() {
+                                    log::error!(
+                                        "Expect handler returned error: {:?}",
+                                        result.err().unwrap()
+                                    );
                                     *this.st = State::Stop;
                                     this = self.as_mut().project();
                                     continue;
@@ -441,11 +446,11 @@ where
                 }
                 // stop io tasks and call upgrade service
                 State::Upgrade(ref mut req) => {
-                    log::trace!("switching to upgrade service");
-
                     let io = this.inner.io.take();
                     let req = req.take().unwrap();
 
+                    log::trace!("switching to upgrade service for {:?}", req);
+
                     // Handle UPGRADE request
                     crate::rt::spawn(this.inner.config.upgrade.as_ref().unwrap().call((
                         req,
