@@ -583,7 +583,7 @@ where
                         c.value().as_bytes(),
                         crate::http::helpers::USERINFO,
                     );
-                    let _ = write!(&mut cookie, "; {}={}", name, value);
+                    let _ = write!(cookie, "; {}={}", name, value);
                 }
                 inner.head.headers.insert(
                     header::COOKIE,
@@ -768,6 +768,14 @@ impl<F: Filter> WsConnection<F> {
     }
 }
 
+impl<F> fmt::Debug for WsConnection<F> {
+    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
+        f.debug_struct("WsConnection")
+            .field("response", &self.res)
+            .finish()
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
