@@ -10,7 +10,6 @@ mod handshake;
 mod mask;
 mod proto;
 mod sink;
-mod stream;
 mod transport;
 
 pub mod error;
@@ -21,5 +20,4 @@ pub use self::frame::Parser;
 pub use self::handshake::{handshake, handshake_response, verify_handshake};
 pub use self::proto::{hash_key, CloseCode, CloseReason, OpCode};
 pub use self::sink::WsSink;
-pub use self::stream::{StreamDecoder, StreamEncoder};
 pub use self::transport::{WsTransport, WsTransportFactory};
