@@ -3,9 +3,8 @@ use std::{io, sync::Arc};
 use ntex::codec::BytesCodec;
 use ntex::connect::Connect;
 use ntex::io::{types::PeerAddr, Io};
-use ntex::server::test_server;
 use ntex::service::{fn_service, pipeline_factory, Service, ServiceFactory};
-use ntex::{time, util::Bytes};
+use ntex::{server::test_server, time, util::Bytes};
 
 #[cfg(feature = "openssl")]
 fn ssl_acceptor() -> tls_openssl::ssl::SslAcceptor {
@@ -72,12 +71,55 @@ mod danger {
 #[ntex::test]
 async fn test_openssl_string() {
     use ntex::server::openssl;
-    use ntex_tls::openssl::PeerCert;
+    use ntex_tls::{openssl::PeerCert, openssl::PeerCertChain, types::HttpProtocol};
     use tls_openssl::{
         ssl::{SslConnector, SslMethod, SslVerifyMode},
         x509::X509,
     };
 
+    let srv = test_server(|| {
+        pipeline_factory(fn_service(|io: Io<_>| async move {
+            let res = io.read_ready().await;
+            assert!(res.is_ok());
+            Ok(io)
+        }))
+        .and_then(openssl::Acceptor::new(ssl_acceptor()))
+        .and_then(fn_service(|io: Io<_>| async move {
+            io.send(Bytes::from_static(b"test"), &BytesCodec)
+                .await
+                .unwrap();
+            assert_eq!(io.recv(&BytesCodec).await.unwrap().unwrap(), "test");
+            Ok::<_, Box<dyn std::error::Error>>(())
+        }))
+    });
+
+    let mut builder = SslConnector::builder(SslMethod::tls()).unwrap();
+    builder.set_verify(SslVerifyMode::NONE);
+
+    let conn = ntex::connect::openssl::Connector::new(builder.build());
+    let addr = format!("127.0.0.1:{}", srv.addr().port());
+    let io = conn.call(addr.into()).await.unwrap();
+    assert_eq!(io.query::<PeerAddr>().get().unwrap(), srv.addr().into());
+    assert_eq!(
+        io.query::<HttpProtocol>().get().unwrap(),
+        HttpProtocol::Http1
+    );
+    let cert = X509::from_pem(include_bytes!("cert.pem")).unwrap();
+    assert_eq!(
+        io.query::<PeerCert>().as_ref().unwrap().0.to_der().unwrap(),
+        cert.to_der().unwrap()
+    );
+    assert_eq!(io.query::<PeerCertChain>().as_ref().unwrap().0.len(), 1);
+    let item = io.recv(&BytesCodec).await.unwrap().unwrap();
+    assert_eq!(item, Bytes::from_static(b"test"));
+}
+
+#[cfg(feature = "openssl")]
+#[ntex::test]
+async fn test_openssl_read_before_error() {
+    use ntex::server::openssl;
+    use tls_openssl::ssl::{SslConnector, SslMethod, SslVerifyMode};
+
     let srv = test_server(|| {
         pipeline_factory(fn_service(|io: Io<_>| async move {
             let res = io.read_ready().await;
@@ -100,45 +142,12 @@ async fn test_openssl_string() {
     let conn = ntex::connect::openssl::Connector::new(builder.build());
     let addr = format!("127.0.0.1:{}", srv.addr().port());
     let io = conn.call(addr.into()).await.unwrap();
-    assert_eq!(io.query::<PeerAddr>().get().unwrap(), srv.addr().into());
-    let cert = X509::from_pem(include_bytes!("cert.pem")).unwrap();
-    assert_eq!(
-        io.query::<PeerCert>().as_ref().unwrap().0.to_der().unwrap(),
-        cert.to_der().unwrap()
-    );
     let item = io.recv(&BytesCodec).await.unwrap().unwrap();
     assert_eq!(item, Bytes::from_static(b"test"));
-}
 
-#[cfg(feature = "openssl")]
-#[ntex::test]
-async fn test_openssl_read_before_error() {
-    use ntex::server::openssl;
-    use tls_openssl::ssl::{SslConnector, SslMethod, SslVerifyMode};
-
-    let srv = test_server(|| {
-        pipeline_factory(fn_service(|io: Io<_>| async move {
-            let res = io.read_ready().await;
-            assert!(res.is_ok());
-            Ok(io)
-        }))
-        .and_then(openssl::Acceptor::new(ssl_acceptor()))
-        .and_then(fn_service(|io: Io<_>| async move {
-            io.send(Bytes::from_static(b"test"), &BytesCodec)
-                .await
-                .unwrap();
-            Ok::<_, Box<dyn std::error::Error>>(())
-        }))
-    });
-
-    let mut builder = SslConnector::builder(SslMethod::tls()).unwrap();
-    builder.set_verify(SslVerifyMode::NONE);
-
-    let conn = ntex::connect::openssl::Connector::new(builder.build());
-    let addr = format!("127.0.0.1:{}", srv.addr().port());
-    let io = conn.call(addr.into()).await.unwrap();
-    let item = io.recv(&BytesCodec).await.unwrap().unwrap();
-    assert_eq!(item, Bytes::from_static(b"test"));
+    io.send(Bytes::from_static(b"test"), &BytesCodec)
+        .await
+        .unwrap();
     assert!(io.recv(&BytesCodec).await.unwrap().is_none());
 }
 
@@ -146,7 +155,7 @@ async fn test_openssl_read_before_error() {
 #[ntex::test]
 async fn test_rustls_string() {
     use ntex::server::rustls;
-    use ntex_tls::rustls::PeerCert;
+    use ntex_tls::{rustls::PeerCert, rustls::PeerCertChain, types::HttpProtocol};
     use rustls_pemfile::certs;
     use std::fs::File;
     use std::io::BufReader;
@@ -160,9 +169,12 @@ async fn test_rustls_string() {
         }))
         .and_then(rustls::Acceptor::new(tls_acceptor()))
         .and_then(fn_service(|io: Io<_>| async move {
+            assert!(io.query::<PeerCert>().as_ref().is_none());
+            assert!(io.query::<PeerCertChain>().as_ref().is_none());
             io.send(Bytes::from_static(b"test"), &BytesCodec)
                 .await
                 .unwrap();
+            assert_eq!(io.recv(&BytesCodec).await.unwrap().unwrap(), "test");
             Ok::<_, std::io::Error>(())
         }))
     });
@@ -176,6 +188,10 @@ async fn test_rustls_string() {
     let addr = format!("localhost:{}", srv.addr().port());
     let io = conn.call(addr.into()).await.unwrap();
     assert_eq!(io.query::<PeerAddr>().get().unwrap(), srv.addr().into());
+    assert_eq!(
+        io.query::<HttpProtocol>().get().unwrap(),
+        HttpProtocol::Http1
+    );
     let cert_file = &mut BufReader::new(File::open("tests/cert.pem").unwrap());
     let cert_chain: Vec<Certificate> = certs(cert_file)
         .unwrap()
@@ -186,8 +202,14 @@ async fn test_rustls_string() {
         io.query::<PeerCert>().as_ref().unwrap().0,
         *cert_chain.first().unwrap()
     );
+    assert_eq!(io.query::<PeerCertChain>().as_ref().unwrap().0, cert_chain);
     let item = io.recv(&BytesCodec).await.unwrap().unwrap();
     assert_eq!(item, Bytes::from_static(b"test"));
+
+    io.send(Bytes::from_static(b"test"), &BytesCodec)
+        .await
+        .unwrap();
+    assert!(io.recv(&BytesCodec).await.unwrap().is_none());
 }
 
 #[ntex::test]
@@ -197,6 +219,7 @@ async fn test_static_str() {
             io.send(Bytes::from_static(b"test"), &BytesCodec)
                 .await
                 .unwrap();
+            time::sleep(time::Millis(100)).await;
             Ok::<_, io::Error>(())
         })
     });
